import 'dart:convert';

import 'arb_resource.dart';


class ArbDocument {
  String locale;

  DateTime lastModified;

  List<ArbResource> entries;

  ArbDocument(this.locale, this.lastModified, this.entries);

  Map<String, Object> toJson() => {
        '_locale': locale,
        '@@last_modified': lastModified.toIso8601String(),
        ...entries.asMap().map((_, ArbResource resource) =>
            MapEntry(resource.id.text, resource.value.text)),
        ...entries
            .where((resource) => resource.hasAttributes)
            .toList()
            .asMap()
            .map((_, ArbResource resource) =>
                MapEntry("@@${resource.id.text}", resource.attributes))
      };

  ArbDocument.fromJson(Map<String, dynamic> _json) {
    var entriesMap = Map<String, ArbResource>();
    entries = List<ArbResource>();

    _json.forEach((key, value) {
      if ("_locale" == key) {
        locale = value;
      } else if ("@@last_modified" == key) {
        lastModified = DateTime.parse(value);
      } else if (key.startsWith("@@")) {
        var entry = entriesMap[key.substring(2)];
        entry.attributes.addAll(value);
      } else {
        var entry = ArbResource(key, value);
        entries.add(entry);
        entriesMap[key] = entry;
      }
    });
  }

  String encode() {
    var encoder = new JsonEncoder.withIndent('  ');
    var arbContent = encoder.convert(this.toJson());
    return arbContent;
  }

  factory ArbDocument.decode(String json) {
    var decoder = new JsonDecoder();
    ArbDocument arbContent = ArbDocument.fromJson(decoder.convert(json));
    return arbContent;
  }

}


