

enum ArbResourceType {
  text
}

class ArbResource {
  final ArbResourceId id;
  final ArbResourceValue value;
  final Map<String, Object> attributes = Map();

  final bool hasAttributes;

  ArbResource(String id, String value, {this.hasAttributes = true})
      : this.id = ArbResourceId(id),
        this.value = ArbResourceValue(value) {
    if (hasAttributes) {
      attributes['type'] = "Text";
      attributes['placeholders'] = Map<String, Object>();
    }
  }
}

class ArbResourceId {
  final String text;

  ArbResourceId(this.text);
}

class ArbResourceValue {
  final String text;
  final placeholders = List<ArbResourcePlaceholder>();

  bool get hasPlaceholders => placeholders.isNotEmpty;

  ArbResourceValue(this.text) {
    var placeholders = findPlaceholders(text);
    if (placeholders.isNotEmpty) {
      this.placeholders.addAll(placeholders);
    }
  }
}

class ArbResourcePlaceholder {
  final String name;

  ArbResourcePlaceholder(this.name);
}

var _regex = RegExp("\\{(.+?)\\}");

List<ArbResourcePlaceholder> findPlaceholders(String text) {
  if (text == null || text.isEmpty) {
    return List<ArbResourcePlaceholder>();
  }

  var matches = _regex.allMatches(text);
  var placeholders = Map<String, ArbResourcePlaceholder>();
  matches.forEach((Match match) {
    var group = match.group(0);
    var placeholder = group.substring(1, group.length - 1);

    if (placeholders.containsKey(placeholder)) {
      throw Exception("Placeholder $placeholder already declared");
    }
    placeholders[placeholder] = (ArbResourcePlaceholder(placeholder));
  });
  return placeholders.values.toList();
}
